import json
import boto3
import os
from datetime import datetime
from dateutil import tz

MQTT_API_NAME = (os.environ.get("MQTT_LAMBDA"))
ES_ACCESS_LAMBDA = (os.environ.get('ES_ACCESS_LAMBDA'))
TENANT_ID = (os.environ.get('TENANT_ID'))

def make_response(code, body):
    response = {
        'statusCode': code,
        "headers": {"Content-Type": "application/json", "Access-Control-Allow-Origin": "*"},
        'body': json.dumps(body),
        "isBase64Encoded": False
    }
    return response

def dictHasKey(dictionary, key):
    return isinstance(dictionary, dict) and key in dictionary

def get_param_from_event(event, paramName):
    if dictHasKey(event, 'queryStringParameters'):
        if dictHasKey(event['queryStringParameters'], paramName):
            return event['queryStringParameters'][paramName]
    return ''

def make_index_name(event):
    JST = tz.gettz('Asia/Tokyo')
    date = datetime.now(JST).strftime('%Y.%m.%d')
    # インデックス名(テナントID_YYYY.MM.DD)を生成
    index_name = '{}_{}'.format(TENANT_ID, date)
    return index_name

def make_query(event):
    query = {
        'query': {
            'bool': {
                'should': []
            }
        },
        'sort': [{'timestamp': 'desc'}],
    }
    # クエリにコネクタIDを設定
    connector_query = {
        'match': {
            'connectorID': get_param_from_event(event, 'connectorId')
        }
    }
    query['query']['bool']['should'].append(connector_query)
    # クエリにページネーション用のページサイズとオフセットを設定
    # 最新のデータを1件取得
    query['from'] = 0
    query['size'] = 1
    # 生成したクエリを返す
    return query

def lambda_handler(event, context):
    # 入力チェック
    index_name = make_index_name(event)
    # クエリ
    query = make_query(event)
    # ペイロード
    payload = {
        'method': 'Get',
        'index': index_name,
        'query': query
    }
    # ESにアクセス
    res = boto3.client('lambda').invoke(
        FunctionName=ES_ACCESS_LAMBDA,
        InvocationType='RequestResponse',
        Payload=json.dumps(payload)
    )
    # レスポンス
    response = json.loads(res['Payload'].read())
    # 最新の環境データを取得
    env = response["body"]["result"]["hits"]["hits"][0]["_source"]
    temp = env["temp"]
    humd = env["humd"]
    # 不快指数を計算
    # https://ja.wikipedia.org/wiki/%E4%B8%8D%E5%BF%AB%E6%8C%87%E6%95%B0

    thi = ...

    res_thi = {"thi": thi}
    edgeID = get_param_from_event(event, 'edgeId')
    payload = {
        'tenantId': TENANT_ID,
        'edgeId': edgeID,
        'payload': res_thi,
        'qos': 1
    }
    # My-IoT edgeにメッセージ送信
    res = boto3.client('lambda').invoke(
        FunctionName=MQTT_API_NAME,
        InvocationType='RequestResponse',
        Payload=json.dumps(payload)
    )
    # API URL呼出に対する返値
    result = {
        'error': 0,
        'message': 'Sent a message to edge.',
        'data': res_thi
    }

    return make_response(response['statusCode'], result)
# EOF